# Generated by Django 3.2.12 on 2022-02-13 17:48

import datetime
from django.db import migrations, models
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Profile',
            fields=[
                ('oid', models.UUIDField(default=uuid.uuid4, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('modified', models.DateTimeField(auto_now=True)),
                ('device_category', models.CharField(max_length=16)),
                ('platform', models.CharField(max_length=16)),
                ('screen_height', models.IntegerField()),
                ('screen_width', models.IntegerField()),
                ('user_agent', models.TextField()),
                ('viewport_height', models.IntegerField()),
                ('viewport_width', models.IntegerField()),
                ('weight', models.DecimalField(decimal_places=24, max_digits=25)),
            ],
            options={
                'db_table': 'djspoofer_profile',
                'ordering': ['-weight'],
            },
        ),
        migrations.CreateModel(
            name='Proxy',
            fields=[
                ('oid', models.UUIDField(default=uuid.uuid4, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('modified', models.DateTimeField(auto_now=True)),
                ('url', models.TextField(unique=True)),
                ('mode', models.IntegerField(choices=[(0, 'ROTATING'), (1, 'STICKY'), (2, 'GENERAL')], default=2)),
                ('country', models.CharField(blank=True, max_length=3, null=True)),
                ('city', models.CharField(blank=True, max_length=64, null=True)),
                ('last_used', models.DateTimeField(blank=True, null=True)),
                ('used_ct', models.IntegerField(default=0)),
                ('cooldown', models.DurationField(default=datetime.timedelta(seconds=600))),
            ],
            options={
                'db_table': 'djspoofer_proxy',
                'ordering': ['url'],
            },
        ),
        migrations.AddIndex(
            model_name='profile',
            index=models.Index(fields=['device_category'], name='profile_device_category_index'),
        ),
        migrations.AddIndex(
            model_name='profile',
            index=models.Index(fields=['platform'], name='profile_platform_index'),
        ),
    ]



import logging
from abc import ABC
from ssl import TLSVersion

import httpx
from djstarter.clients import Http2Client

from . import utils
from .models import Fingerprint, TLSFingerprint

logger = logging.getLogger(__name__)


class DesktopClient(ABC, Http2Client):
    def __init__(self, fingerprint=None, proxy=None, *args, **kwargs):
        self.fingerprint = fingerprint or self.temp_fingerprint()
        self.proxy = proxy
        self.tls_fingerprint = self.fingerprint.tls_fingerprint or self.generate_tls_fingerprint()
        self.user_agent = self.fingerprint.user_agent
        super().__init__(
            proxies=self.proxies,
            verify=self.new_ssl_context(),
            *args,
            **kwargs
        )

    def send(self, *args, **kwargs):
        self.headers.pop('Accept-Encoding', None)
        self.headers.pop('Connection', None)
        return super().send(*args, **kwargs)

    def new_ssl_context(self):
        context = httpx.create_ssl_context(http2=True)
        context.minimum_version = TLSVersion.TLSv1_2
        context.set_ciphers(self.tls_fingerprint.ciphers)
        context.options = self.tls_fingerprint.extensions

        return context

    @property
    def proxies(self):
        if self.proxy:
            return {
                'http://': self.proxy.http_url,
                'https://': self.proxy.http_url
            }
        return dict()

    @staticmethod
    def temp_fingerprint():
        return Fingerprint.objects.get_random_desktop_fingerprint()

    def generate_tls_fingerprint(self):
        tls_fingerprint = TLSFingerprint.objects.create(browser=self.fingerprint.browser)
        self.fingerprint.tls_fingerprint = tls_fingerprint
        self.fingerprint.save()
        return tls_fingerprint


class DesktopChromeClient(DesktopClient):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.ua_parser = utils.UserAgentParser(self.user_agent)

    def init_headers(self):
        return {
            'user-agent': self.user_agent,
        }

    @property
    def sec_ch_ua(self):
        version = self.ua_parser.browser_major_version
        return f'" Not;A Brand";v="99", "Google Chrome";v="{version}", "Chromium";v="{version}"'

    @property
    def sec_ch_ua_mobile(self):
        return '?0'

    @property
    def sec_ch_ua_platform(self):
        platform = self.ua_parser.os
        return f'"{platform}"'


class DesktopFirefoxClient(DesktopClient):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def init_headers(self):
        return {
            'User-Agent': self.user_agent,
        }



# Generated by Django 3.2.12 on 2022-04-08 14:37

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djspoofer', '0013_auto_20220407_2022'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='fingerprint',
            name='proxy',
        ),
    ]



# Generated by Django 3.2.12 on 2022-04-03 22:52

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djspoofer', '0007_auto_20220330_2108'),
    ]

    operations = [
        migrations.AddField(
            model_name='tlsfingerprint',
            name='user_agent',
            field=models.TextField(default=''),
            preserve_default=False,
        ),
    ]



from django.test import TestCase

from djspoofer import providers
from djspoofer.providers import UsernameProvider


class UsernameProviderTests(TestCase):
    """
    Username Provider Tests
    """

    @classmethod
    def setUpTestData(cls):
        cls.username = 'username_123'
        cls.test_str = 'Lorem ipsum dolor sit amet, consectetur adipiscing elit.'

    def test_generate_digits(self):
        val = providers.generate_digits(n_digits=14)
        self.assertTrue(len(str(val)) == 14)

        val = providers.generate_digits(n_digits=99)
        self.assertTrue(len(str(val)) == 99)

    def test_replace_character(self):
        val = providers.replace_character(self.test_str, 15, char='X')
        self.assertEquals(val, 'Lorem ipsum dolXr sit amet, consectetur adipiscing elit.')

    def test_reverse(self):
        self.assertEquals(UsernameProvider.reverse(self.username), '321_emanresu')

    def test_repeat(self):
        self.assertEquals(UsernameProvider.repeat(self.username, chance=0), 'username_123')
        self.assertEquals(UsernameProvider.repeat(self.username, chance=1), 'uusseerrnnaammee__112233')

    def test_replace(self):
        self.assertEquals(UsernameProvider.replace(self.username, chance=0), 'username_123')
        self.assertNotEquals(UsernameProvider.replace(self.username, chance=1), 'username_123')

    def test_concat(self):
        word_2 = 'hello'
        new_username = UsernameProvider.concat(self.username, word_2, chance=1)
        self.assertTrue(new_username.startswith(self.username.capitalize()))
        self.assertTrue(new_username.endswith('Hello'))

        word_2 = 'hello'
        new_username = UsernameProvider.concat(self.username, word_2, chance=0)
        self.assertTrue(new_username.startswith(self.username))
        self.assertTrue(new_username.endswith('hello'))

    def test_vowel_remove(self):
        new_username = UsernameProvider.vowel_remove(self.username, chance=0)
        self.assertEquals(new_username, self.username)

        new_username = UsernameProvider.vowel_remove(self.username, chance=1)
        self.assertEquals(new_username, 'srnm_123')

    def test_concat_digits(self):
        new_username = UsernameProvider.concat_digits(self.username, chance=1)
        self.assertIn(len(new_username), [13, 14])

    def test_decorate(self):
        new_username = UsernameProvider.decorate(self.username, chance=0, min_chars=0, max_chars=0)
        self.assertEquals(new_username, self.username)

        new_username = UsernameProvider.decorate(self.username, chance=1, min_chars=3, max_chars=3)
        self.assertEquals(len(new_username), 18)

    def test_leet(self):
        new_username = UsernameProvider.leet(self.username, chance=0)
        self.assertEquals(new_username, self.username)

        new_username = UsernameProvider.leet(self.username, chance=1)
        self.assertEquals(new_username, 'u53rn4m3_123')

    def test_honorific(self):
        new_username = UsernameProvider.honorific(self.username, chance=0)
        self.assertFalse(new_username[0].isupper())

        new_username = UsernameProvider.honorific(self.username, chance=1)
        self.assertTrue(new_username[0].isupper())



# Generated by Django 3.2.12 on 2022-02-20 17:17

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djspoofer', '0003_auto_20220220_1504'),
    ]

    operations = [
        migrations.AlterField(
            model_name='proxy',
            name='mode',
            field=models.IntegerField(choices=[(10, 'GENERAL'), (20, 'ROTATING'), (30, 'STICKY')], default=10),
        ),
    ]



# Generated by Django 3.2.12 on 2022-04-06 00:36

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djspoofer', '0010_auto_20220404_1546'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='tlsfingerprint',
            name='fingerprint',
        ),
        migrations.RemoveField(
            model_name='fingerprint',
            name='proxy',
        ),
        migrations.AddField(
            model_name='fingerprint',
            name='ciphers',
            field=models.TextField(default=''),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='fingerprint',
            name='extensions',
            field=models.IntegerField(default=1),
            preserve_default=False,
        ),
        migrations.AddIndex(
            model_name='fingerprint',
            index=models.Index(fields=['extensions'], name='fp_extensions_index'),
        ),
        migrations.AddIndex(
            model_name='fingerprint',
            index=models.Index(fields=['ciphers'], name='fp_ciphers_index'),
        ),
        migrations.DeleteModel(
            name='TLSFingerprint',
        ),
    ]



# Generated by Django 3.2.12 on 2022-04-04 15:46

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djspoofer', '0009_remove_tlsfingerprint_user_agent'),
    ]

    operations = [
        migrations.AddField(
            model_name='fingerprint',
            name='browser',
            field=models.CharField(default='', max_length=32),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='fingerprint',
            name='os',
            field=models.CharField(default='', max_length=32),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='fingerprint',
            name='device_category',
            field=models.CharField(max_length=32),
        ),
        migrations.AlterField(
            model_name='fingerprint',
            name='platform',
            field=models.CharField(max_length=32),
        ),
    ]



from ssl import Options

from django.test import TestCase

from djspoofer.models import Fingerprint, Proxy, TLSFingerprint
from djspoofer import utils


class FingerprintTests(TestCase):
    """
    Fingerprint Tests
    """

    @classmethod
    def setUpTestData(cls):
        user_agent = 'Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:98.0) Gecko/20100101 Firefox/98.0'
        ua_parser = utils.UserAgentParser(user_agent)
        cls.fingerprint_data = {
            'browser': ua_parser.browser,
            'device_category': 'mobile',
            'platform': 'US',
            'screen_height': 1920,
            'screen_width': 1080,
            'user_agent': ('Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) '
                           'Chrome/99.0.4844.74 Safari/537.36'),
            'viewport_height': 768,
            'viewport_width': 1024,
        }

    def test_user_str(self):
        fp = Fingerprint.objects.create(**self.fingerprint_data)
        self.assertEqual(str(fp), f'Fingerprint -> user_agent: {self.fingerprint_data["user_agent"]}')


class TLSFingerprintTests(TestCase):
    """
    TLSFingerprint Tests
    """

    @classmethod
    def setUpTestData(cls):
        user_agent = 'Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:98.0) Gecko/20100101 Firefox/98.0'
        ua_parser = utils.UserAgentParser(user_agent)
        cls.tls_fingerprint_data = {
            'browser': ua_parser.browser,
        }

    def test_ciphers(self):
        tls_fp = TLSFingerprint.objects.create(**self.tls_fingerprint_data)
        self.assertTrue(':' in tls_fp.ciphers)

    def test_extensions(self):
        tls_fp = TLSFingerprint.objects.create(**self.tls_fingerprint_data)
        self.assertEquals(type(tls_fp.extensions), int)

        tls_fp.extensions = int(Options.OP_NO_TICKET | Options.OP_NO_RENEGOTIATION | Options.OP_ENABLE_MIDDLEBOX_COMPAT)
        self.assertEquals(tls_fp.extensions, 1074806784)


class ProxyTests(TestCase):
    """
    Proxy Tests
    """

    @classmethod
    def setUpTestData(cls):
        cls.proxy_data = {
            'url': 'user123:password456@example.com:4582',
            'country': 'US',
            'city': 'dallas',
        }

    def test_user_str(self):
        proxy = Proxy.objects.create(**self.proxy_data)
        self.assertEqual(str(proxy), 'Proxy -> url: user123:password456@example.com:4582, mode: GENERAL')

    def test_is_on_cooldown(self):
        proxy = Proxy.objects.create(**self.proxy_data)
        self.assertFalse(proxy.is_on_cooldown)

        proxy.set_last_used()
        self.assertTrue(proxy.is_on_cooldown)

    def test_set_last_used(self):
        proxy = Proxy.objects.create(**self.proxy_data)
        self.assertEquals(proxy.used_count, 0)
        self.assertIsNone(proxy.last_used)

        proxy.set_last_used()
        self.assertEquals(proxy.used_count, 1)
        self.assertIsNotNone(proxy.last_used)

    def test_http_url(self):
        proxy = Proxy.objects.create(**self.proxy_data)
        self.assertTrue(proxy.http_url, 'http://user123:password456@example.com:4582')

    def test_https_url(self):
        proxy = Proxy.objects.create(**self.proxy_data)
        self.assertEquals(proxy.https_url, 'https://user123:password456@example.com:4582')




from django.core.management.base import BaseCommand

from djspoofer import const
from djspoofer.models import Proxy


class Command(BaseCommand):
    help = 'Add Proxy'

    def add_arguments(self, parser):
        parser.add_argument(
            "--mode",
            choices=const.ProxyModes.labels(),
            required=True,
            type=str,
            help="Sets the proxy mode",
        )
        parser.add_argument(
            "--url",
            required=True,
            type=str,
            help="Set the proxy url",
        )
        parser.add_argument(
            "--country",
            default='',
            type=str,
            help="Set the proxy country",
        )
        parser.add_argument(
            "--city",
            default='',
            type=str,
            help="Set the proxy city",
        )

    def handle(self, *args, **kwargs):
        try:
            proxy = Proxy.objects.create(
                mode=const.ProxyModes.by_label(kwargs['mode']),
                url=kwargs['url'],
                country=kwargs['country'],
                city=kwargs['city'],
            )
        except Exception as e:
            self.stdout.write(self.style.ERROR(f'Error while running command:\n{str(e)}'))
            raise e
        else:
            self.stdout.write(self.style.MIGRATE_LABEL(f'Created proxy "{proxy.url}"'))



# Generated by Django 3.2.12 on 2022-02-18 15:56

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djspoofer', '0001_initial'),
    ]

    operations = [
        migrations.RenameField(
            model_name='proxy',
            old_name='used_ct',
            new_name='used_count',
        ),
    ]



from django.core.management.base import BaseCommand

from djspoofer.clients import DesktopChromeClient
from incolumitas import incolumitas_tls_api
from djstarter import utils


class Command(BaseCommand):
    help = 'Get TLS Fingerprint'

    def handle(self, *args, **kwargs):
        try:
            with DesktopChromeClient() as client:
                r_tls = incolumitas_tls_api.fps(client)
        except Exception as e:
            self.stdout.write(self.style.ERROR(f'Error while running command:\n{str(e)}'))
            raise e
        else:
            self.stdout.write(utils.pretty_dict(r_tls))
            self.stdout.write(self.style.MIGRATE_LABEL('Finished getting TLS Fingerprint'))



from django.core.management.base import BaseCommand
from djstarter import utils

from djspoofer.clients import DesktopChromeClient
from djspoofer.models import Proxy
from ja3er import ja3er_api


class Command(BaseCommand):
    help = 'Ja3er Check'

    def handle(self, *args, **kwargs):
        try:
            with DesktopChromeClient(proxy=Proxy.objects.get_sticky_proxy()) as chrome_client:
                self.get_ja3_details(chrome_client)
        except Exception as e:
            self.stdout.write(self.style.ERROR(f'Error while running command:\n{str(e)}'))
            raise e
        else:
            self.stdout.write(self.style.MIGRATE_LABEL(f'Successfully got ja3er details'))

    def get_ja3_details(self, chrome_client):
        self.stdout.write(f'Spoofed User Agent: {chrome_client.user_agent}')

        self.stdout.write(utils.eye_catcher_line('JA3 Details'))
        r_json = ja3er_api.get_json(chrome_client)
        self.stdout.write(utils.pretty_dict(r_json.data))

        self.stdout.write(f'ssl_version: {r_json.ssl_version}')
        self.stdout.write(f'ciphers: {r_json.ciphers}')
        self.stdout.write(f'ssl_extensions: {r_json.ssl_extensions}')
        self.stdout.write(f'elliptic_curve: {r_json.elliptic_curve}')
        self.stdout.write(f'elliptic_curve_point_format: {r_json.elliptic_curve_point_format}')

        self.stdout.write(utils.eye_catcher_line('JA3 Hash Search'))
        r_search = ja3er_api.search(chrome_client, ja3_hash=r_json.ja3_hash)
        self.stdout.write(utils.pretty_dict(r_search.json()))






from ssl import Options

from djstarter import utils as core_utils


class ProxyModes(core_utils.ChoiceEnum):
    GENERAL = 10
    ROTATING = 20
    STICKY = 30


class Ciphers:
    TLS_AES_128_GCM_SHA256 = 'AES128-GCM-SHA256'
    TLS_CHACHA20_POLY1305_SHA256 = 'CHACHA20-POLY1305-SHA256'
    TLS_AES_256_GCM_SHA384 = 'AES256-GCM-SHA384'
    TLS_ECDHE_ECDSA_WITH_AES_128_GCM_SHA256 = 'ECDHE-ECDSA-AES128-GCM-SHA256'
    TLS_ECDHE_RSA_WITH_AES_128_GCM_SHA256 = 'ECDHE-RSA-AES128-GCM-SHA256'
    TLS_ECDHE_ECDSA_WITH_CHACHA20_POLY1305_SHA256 = 'ECDHE-ECDSA-CHACHA20-POLY1305'
    TLS_ECDHE_RSA_WITH_CHACHA20_POLY1305_SHA256 = 'ECDHE-RSA-CHACHA20-POLY1305'
    TLS_ECDHE_ECDSA_WITH_AES_256_GCM_SHA384 = 'ECDHE-ECDSA-AES256-GCM-SHA384'
    TLS_ECDHE_RSA_WITH_AES_256_GCM_SHA384 = 'ECDHE-RSA-AES256-GCM-SHA384'
    TLS_ECDHE_ECDSA_WITH_AES_256_CBC_SHA = 'ECDHE-ECDSA-AES256-SHA'
    TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA = 'ECDHE-ECDSA-AES128-SHA'
    TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA = 'ECDHE-RSA-AES128-SHA'
    TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA = 'ECDHE-RSA-AES256-SHA'
    TLS_RSA_WITH_AES_128_GCM_SHA256 = 'AES128-GCM-SHA256'
    TLS_RSA_WITH_AES_256_GCM_SHA384 = 'AES256-GCM-SHA384'
    TLS_RSA_WITH_AES_128_CBC_SHA = 'AES128-SHA'
    TLS_RSA_WITH_AES_256_CBC_SHA = 'AES256-SHA'


ChromeDesktopCiphers = [
    Ciphers.TLS_AES_128_GCM_SHA256,
    Ciphers.TLS_AES_256_GCM_SHA384,
    Ciphers.TLS_CHACHA20_POLY1305_SHA256,
    Ciphers.TLS_ECDHE_ECDSA_WITH_AES_128_GCM_SHA256,
    Ciphers.TLS_ECDHE_RSA_WITH_AES_128_GCM_SHA256,
    Ciphers.TLS_ECDHE_ECDSA_WITH_AES_256_GCM_SHA384,
    Ciphers.TLS_ECDHE_RSA_WITH_AES_256_GCM_SHA384,
    Ciphers.TLS_ECDHE_ECDSA_WITH_CHACHA20_POLY1305_SHA256,
    Ciphers.TLS_ECDHE_RSA_WITH_CHACHA20_POLY1305_SHA256,
    Ciphers.TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA,
    Ciphers.TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA,
    Ciphers.TLS_RSA_WITH_AES_128_GCM_SHA256,
    Ciphers.TLS_RSA_WITH_AES_256_GCM_SHA384,
    Ciphers.TLS_RSA_WITH_AES_128_CBC_SHA,
    Ciphers.TLS_RSA_WITH_AES_256_CBC_SHA,
]

FirefoxDesktopCiphers = [
    Ciphers.TLS_AES_128_GCM_SHA256,
    Ciphers.TLS_CHACHA20_POLY1305_SHA256,
    Ciphers.TLS_AES_256_GCM_SHA384,
    Ciphers.TLS_ECDHE_ECDSA_WITH_AES_128_GCM_SHA256,
    Ciphers.TLS_ECDHE_RSA_WITH_AES_128_GCM_SHA256,
    Ciphers.TLS_ECDHE_ECDSA_WITH_CHACHA20_POLY1305_SHA256,
    Ciphers.TLS_ECDHE_RSA_WITH_CHACHA20_POLY1305_SHA256,
    Ciphers.TLS_ECDHE_ECDSA_WITH_AES_256_GCM_SHA384,
    Ciphers.TLS_ECDHE_RSA_WITH_AES_256_GCM_SHA384,
    Ciphers.TLS_ECDHE_ECDSA_WITH_AES_256_CBC_SHA,
    Ciphers.TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA,
    Ciphers.TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA,
    Ciphers.TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA,
    Ciphers.TLS_RSA_WITH_AES_128_GCM_SHA256,
    Ciphers.TLS_RSA_WITH_AES_256_GCM_SHA384,
    Ciphers.TLS_RSA_WITH_AES_128_CBC_SHA,
]

TLS_EXTENSIONS = [
    Options.OP_CIPHER_SERVER_PREFERENCE,
    Options.OP_SINGLE_DH_USE,
    Options.OP_SINGLE_ECDH_USE,
    Options.OP_NO_COMPRESSION,
    Options.OP_NO_TICKET,
    Options.OP_NO_RENEGOTIATION,
    Options.OP_ENABLE_MIDDLEBOX_COMPAT,
]

SUPPORTED_BROWSERS = (
    'Chrome',
    'Firefox',
)


class OsTTL:
    LINUX = 64
    WINDOWS = 128   # TODO Test This
    MACOS = 64


class OsMSSRecv:
    LINUX = 64240
    WINDOWS = 8192  # TODO Test this



from django.core.management.base import BaseCommand
from djstarter import utils

from djspoofer.clients import DesktopChromeClient
from djspoofer.models import Proxy
from incolumitas import incolumitas_api


class Command(BaseCommand):
    help = 'Get IP Fingerprint'

    def add_arguments(self, parser):
        parser.add_argument(
            "--ip_addr",
            required=False,
            type=str,
            help="Set the proxy url",
        )

    def handle(self, *args, **kwargs):
        try:
            with DesktopChromeClient(proxy=Proxy.objects.get_sticky_proxy()) as client:
                r_tls = incolumitas_api.get_ip_fingerprint(client, ip_addr=kwargs.get('ip_addr'))
        except Exception as e:
            self.stdout.write(self.style.ERROR(f'Error while running command:\n{str(e)}'))
            raise e
        else:
            self.stdout.write(utils.pretty_dict(r_tls))
            self.stdout.write(self.style.MIGRATE_LABEL('Finished getting TLS Fingerprint'))



from unittest import mock

from django.test import TestCase
from httpx import Request, Response, codes

from djspoofer import clients, utils
from djspoofer.models import Fingerprint, TLSFingerprint, Proxy


class DesktopChromeClientTests(TestCase):
    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()
        cls.request = Request(url='', method='')  # Must add a non null request to avoid raising Runtime exception
        cls.mocked_sleep = mock.patch('time.sleep', return_value=None).start()
        cls.proxy = Proxy.objects.create_sticky_proxy(
            url='test123:5000',
        )
        user_agent = 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/99.0.4844.82 Safari/537.36'
        ua_parser = utils.UserAgentParser(user_agent)
        Fingerprint.objects.create(
            browser=ua_parser.browser,
            device_category='desktop',
            os=ua_parser.os,
            platform='US',
            screen_height=1920,
            screen_width=1080,
            user_agent=user_agent,
            viewport_height=768,
            viewport_width=1024,
            tls_fingerprint=TLSFingerprint.objects.create(
                browser=ua_parser.browser,
            )
        )

    @mock.patch.object(clients.DesktopChromeClient, '_send_handling_auth')
    def test_ok(self, mock_sd_send):
        mock_sd_send.return_value = Response(
            request=self.request,
            status_code=codes.OK,
            text='ok'
        )
        with clients.DesktopChromeClient(proxy=self.proxy) as chrome_client:
            chrome_client.get('http://example.com')
            self.assertEquals(mock_sd_send.call_count, 1)
            self.assertEquals(
                chrome_client.sec_ch_ua,
                '" Not;A Brand";v="99", "Google Chrome";v="99", "Chromium";v="99"'
            )
            self.assertEquals(chrome_client.sec_ch_ua_mobile, '?0')
            self.assertEquals(chrome_client.sec_ch_ua_platform, '"Windows"')


class DesktopFirefoxClientTests(TestCase):
    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()
        cls.request = Request(url='', method='')  # Must add a non null request to avoid raising Runtime exception
        cls.mocked_sleep = mock.patch('time.sleep', return_value=None).start()

        user_agent = 'Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:98.0) Gecko/20100101 Firefox/98.0'
        ua_parser = utils.UserAgentParser(user_agent)
        Fingerprint.objects.create(
            browser=ua_parser.browser,
            device_category='desktop',
            os=ua_parser.os,
            platform='US',
            screen_height=1920,
            screen_width=1080,
            user_agent=user_agent,
            viewport_height=768,
            viewport_width=1024,
            tls_fingerprint=TLSFingerprint.objects.create(
                browser=ua_parser.browser,
            )
        )

    @mock.patch.object(clients.DesktopFirefoxClient, '_send_handling_auth')
    def test_ok(self, mock_sd_send):
        mock_sd_send.return_value = Response(
            request=self.request,
            status_code=codes.OK,
            text='ok'
        )
        with clients.DesktopFirefoxClient() as sd_client:
            sd_client.get('http://example.com')
            self.assertEquals(mock_sd_send.call_count, 1)





from django.core.management.base import BaseCommand

from djspoofer.models import Fingerprint, TLSFingerprint
from djspoofer import utils
from intoli.models import Profile


class Command(BaseCommand):
    help = 'Create Desktop Fingerprints'

    def add_arguments(self, parser):
        parser.add_argument(
            "--num_to_create",
            required=True,
            type=int,
            help="Number of Fingerprints to Create",
        )

    def handle(self, *args, **kwargs):
        try:
            self.create_all_fingerprints(kwargs['num_to_create'])
        except Exception as e:
            self.stdout.write(self.style.ERROR(f'Error while running command:\n{str(e)}'))
            raise e
        else:
            self.stdout.write(self.style.MIGRATE_LABEL(f'Successfully created fingerprints'))

    def create_all_fingerprints(self, num_to_create):
        fingerprints = list()
        for _ in range(num_to_create):
            fingerprints.append(self.create_fingerprint())
        self.stdout.write(self.style.MIGRATE_LABEL(f'Fingerprints created: {len(fingerprints)}'))

    @staticmethod
    def create_fingerprint():
        profile = Profile.objects.random_desktop_profile()
        ua_parser = utils.UserAgentParser(profile.user_agent)
        return Fingerprint.objects.create(
            browser=ua_parser.browser,
            device_category=profile.device_category,
            os=ua_parser.os,
            platform=profile.platform,
            screen_height=profile.screen_height,
            screen_width=profile.screen_width,
            user_agent=profile.user_agent,
            viewport_height=profile.viewport_height,
            viewport_width=profile.viewport_width,
            tls_fingerprint=TLSFingerprint.objects.create(
                browser=ua_parser.browser
            )
        )



# Generated by Django 3.2.12 on 2022-03-30 18:00

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djspoofer', '0005_auto_20220328_2337'),
    ]

    operations = [
        migrations.AddField(
            model_name='tlsfingerprint',
            name='client',
            field=models.IntegerField(choices=[(5, 'GENERIC'), (20, 'CHROME_DESKTOP'), (30, 'FIREFOX_DESKTOP')], default=5),
        ),
        migrations.AddIndex(
            model_name='tlsfingerprint',
            index=models.Index(fields=['client'], name='tls_fp_client_index'),
        ),
    ]



import datetime
import random

from django.db import models
from django.utils import timezone
from djstarter.models import BaseModel

from . import const, managers


class Proxy(BaseModel):
    objects = managers.ProxyManager()

    url = models.TextField(unique=True, blank=False)
    mode = models.IntegerField(default=const.ProxyModes.GENERAL.value, choices=const.ProxyModes.choices())
    country = models.CharField(max_length=3, blank=True, null=True)
    city = models.CharField(max_length=64, blank=True, null=True)
    last_used = models.DateTimeField(blank=True, null=True)
    used_count = models.IntegerField(default=0)
    cooldown = models.DurationField(default=datetime.timedelta(minutes=10))

    class Meta:
        db_table = 'djspoofer_proxy'
        ordering = ['url']
        app_label = 'djspoofer'

    def __str__(self):
        return f'Proxy -> url: {self.url}, mode: {self.pretty_mode}'

    @property
    def http_url(self):
        return f'http://{self.url}'

    @property
    def https_url(self):
        return f'https://{self.url}'

    @property
    def auth(self):
        return self.url.split('@')[0] if '@' in self.url else None

    @property
    def auth_username(self):
        return self.auth.split(':')[0] if self.auth else None

    @property
    def auth_password(self):
        return self.auth.split(':')[1] if self.auth else None

    @property
    def is_on_cooldown(self):
        if self.last_used:
            return self.last_used > timezone.now() - self.cooldown
        return False

    @property
    def pretty_mode(self):
        return self.get_mode_display()

    def set_last_used(self):
        self.last_used = timezone.now()
        self.used_count += 1
        self.save()


class TLSFingerprint(BaseModel):
    objects = managers.FingerprintManager()

    browser = models.CharField(max_length=32)
    extensions = models.IntegerField()
    ciphers = models.TextField()

    class Meta:
        db_table = 'djspoofer_tls_fingerprint'
        ordering = ['-created']
        app_label = 'djspoofer'

        indexes = [
            models.Index(fields=['browser', ], name='tls_fp_browser_index'),
        ]

    def generate_chrome_desktop_cipher_str(self):
        grease_cipher = f'TLS_GREASE_IS_THE_WORD_{random.randint(1, 8)}A'
        return ':'.join(
            [grease_cipher] + [c for c in self.shuffled_ciphers(ciphers=const.ChromeDesktopCiphers, start_idx=4)]
        )

    def generate_firefox_desktop_cipher_str(self):
        return ':'.join([c for c in self.shuffled_ciphers(ciphers=const.FirefoxDesktopCiphers, start_idx=3)])

    DESKTOP_CLIENT_CIPHER_MAP = {
        'Chrome': generate_chrome_desktop_cipher_str,
        'Firefox': generate_firefox_desktop_cipher_str
    }

    @staticmethod
    def shuffled_ciphers(ciphers, start_idx=0, min_k=6):
        first_ciphers = ciphers[:start_idx]
        rem_ciphers = ciphers[start_idx:]
        k = random.randint(min_k, len(rem_ciphers))
        return first_ciphers + random.sample(rem_ciphers, k=k)

    @staticmethod
    def random_tls_extension_int(min_k=4):
        k = random.randint(min_k, len(const.TLS_EXTENSIONS))
        ext_val = 0
        for ext in random.sample(const.TLS_EXTENSIONS, k=k):
            ext_val |= ext
        return int(ext_val)

    def save(self, *args, **kwargs):
        if not self.ciphers:
            self.ciphers = self.DESKTOP_CLIENT_CIPHER_MAP[self.browser](self)
        if not self.extensions:
            self.extensions = self.random_tls_extension_int()
        super().save(*args, **kwargs)


class Fingerprint(BaseModel):
    objects = managers.FingerprintManager()

    browser = models.CharField(max_length=32)
    device_category = models.CharField(max_length=32)
    os = models.CharField(max_length=32)
    platform = models.CharField(max_length=32)
    screen_height = models.IntegerField()
    screen_width = models.IntegerField()
    user_agent = models.TextField()
    viewport_height = models.IntegerField()
    viewport_width = models.IntegerField()

    tls_fingerprint = models.ForeignKey(
        to=TLSFingerprint,
        related_name='fingerprints',
        on_delete=models.CASCADE,
        blank=True,
        null=True
    )

    class Meta:
        db_table = 'djspoofer_fingerprint'
        ordering = ['-created']
        app_label = 'djspoofer'

        indexes = [
            models.Index(fields=['browser', ], name='fp_browser_index'),
            models.Index(fields=['device_category', ], name='fp_device_category_index'),
            models.Index(fields=['platform', ], name='fp_platform_index'),
        ]

    def __str__(self):
        return f'Fingerprint -> user_agent: {self.user_agent}'





import httpx

httpx.HTTPTransport
class FingerprintTransport(httpx.BaseTransport):
    """
    Spoofs TCPIP Fingerprints
    """

    def handle_request(self, request):
        print('YAYAYA')
        return super().handle_request(request)



# Generated by Django 3.2.12 on 2022-02-20 15:04

from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('djspoofer', '0002_rename_used_ct_proxy_used_count'),
    ]

    operations = [
        migrations.CreateModel(
            name='Fingerprint',
            fields=[
                ('oid', models.UUIDField(default=uuid.uuid4, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('modified', models.DateTimeField(auto_now=True)),
                ('device_category', models.CharField(max_length=16)),
                ('platform', models.CharField(max_length=16)),
                ('screen_height', models.IntegerField()),
                ('screen_width', models.IntegerField()),
                ('user_agent', models.TextField()),
                ('viewport_height', models.IntegerField()),
                ('viewport_width', models.IntegerField()),
                ('proxy', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='fingerprints', to='djspoofer.proxy')),
            ],
            options={
                'db_table': 'djspoofer_fingerprint',
                'ordering': ['-created'],
            },
        ),
        migrations.DeleteModel(
            name='Profile',
        ),
        migrations.AddIndex(
            model_name='fingerprint',
            index=models.Index(fields=['device_category'], name='fp_device_category_index'),
        ),
        migrations.AddIndex(
            model_name='fingerprint',
            index=models.Index(fields=['platform'], name='fp_platform_index'),
        ),
    ]



# Generated by Django 3.2.12 on 2022-04-07 20:22

from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('djspoofer', '0012_fingerprint_proxy'),
    ]

    operations = [
        migrations.CreateModel(
            name='TLSFingerprint',
            fields=[
                ('oid', models.UUIDField(default=uuid.uuid4, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('modified', models.DateTimeField(auto_now=True)),
                ('browser', models.CharField(max_length=32)),
                ('extensions', models.IntegerField()),
                ('ciphers', models.TextField()),
            ],
            options={
                'db_table': 'djspoofer_tls_fingerprint',
                'ordering': ['-created'],
            },
        ),
        migrations.RemoveIndex(
            model_name='fingerprint',
            name='fp_extensions_index',
        ),
        migrations.RemoveIndex(
            model_name='fingerprint',
            name='fp_ciphers_index',
        ),
        migrations.RemoveField(
            model_name='fingerprint',
            name='ciphers',
        ),
        migrations.RemoveField(
            model_name='fingerprint',
            name='extensions',
        ),
        migrations.AddIndex(
            model_name='fingerprint',
            index=models.Index(fields=['browser'], name='fp_browser_index'),
        ),
        migrations.AddIndex(
            model_name='tlsfingerprint',
            index=models.Index(fields=['browser'], name='tls_fp_browser_index'),
        ),
        migrations.AddField(
            model_name='fingerprint',
            name='tls_fingerprint',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='fingerprints', to='djspoofer.tlsfingerprint'),
        ),
    ]



# Generated by Django 3.2.12 on 2022-04-03 22:53

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djspoofer', '0008_tlsfingerprint_user_agent'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='tlsfingerprint',
            name='user_agent',
        ),
    ]



# Generated by Django 3.2.12 on 2022-03-30 21:08

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('djspoofer', '0006_auto_20220330_1800'),
    ]

    operations = [
        migrations.RemoveIndex(
            model_name='tlsfingerprint',
            name='tls_fp_client_index',
        ),
        migrations.RemoveField(
            model_name='fingerprint',
            name='tls_fingerprint',
        ),
        migrations.RemoveField(
            model_name='tlsfingerprint',
            name='client',
        ),
        migrations.AddField(
            model_name='tlsfingerprint',
            name='fingerprint',
            field=models.OneToOneField(default='', on_delete=django.db.models.deletion.CASCADE, related_name='tls_fingerprint', to='djspoofer.fingerprint'),
            preserve_default=False,
        ),
    ]



import logging
import random

from faker import Faker
from ua_parser import user_agent_parser

from . import providers

logger = logging.getLogger(__name__)


fake = Faker('en_US')
fake.add_provider(providers.UsernameProvider)
fake.add_provider(providers.PhoneNumberProvider)


class FakeProfile:
    MIN_PWD_LEN = 6

    def __init__(self, username=None):
        self.username = username or fake.username()
        self.gender = random.choice(['M', 'F'])
        self.first_name = fake.first_name_male() if self.gender == 'M' else fake.first_name_female()
        self.last_name = fake.last_name()
        self.dob = fake.date_of_birth(minimum_age=18, maximum_age=60)
        self.contact_email = f'{fake.username()}@{fake.free_email_domain()}'
        self.addr_state = fake.state_abbr()
        self.us_phone_number = fake.us_e164()
        self.password = fake.password(length=12, special_chars=True, digits=True, upper_case=True, lower_case=True)

    def __str__(self):
        return f'FakeProfile -> username: {self.username}, full_name: {self.full_name}'

    @property
    def full_gender(self):
        return 'MALE' if self.gender == 'M' else 'FEMALE'

    @property
    def full_name(self):
        return f'{self.first_name} {self.last_name}'

    @property
    def dob_yyyymmdd(self):
        return self.dob.strftime('%Y-%m-%d')


class UserAgentParser:
    class UserAgent:
        def __init__(self, data):
            self.data = data

        @property
        def family(self):
            return self.data['family']

        @property
        def major(self):
            return self.data['major']

        @property
        def minor(self):
            return self.data['minor']

        @property
        def patch(self):
            return self.data['patch']

    class OS(UserAgent):
        pass

    def __init__(self, user_agent):
        self.ua_parser = user_agent_parser.Parse(user_agent)
        self._user_agent = self.UserAgent(self.ua_parser['user_agent'])
        self._os = self.OS(self.ua_parser['os'])

    @property
    def browser(self):
        return self._user_agent.family

    @property
    def browser_major_version(self):
        return self._user_agent.major

    @property
    def os(self):
        return self._os.family

    def __str__(self):
        return f'UserAgentParser -> {self.ua_parser}'



from django.db import models
from django.db import transaction
from django.db.models import F, Q
from django.utils import timezone

from djspoofer import exceptions
from . import const


class FingerprintManager(models.Manager):
    def all_desktop_profiles(self):
        return super().get_queryset().filter(device_category='desktop', browser__in=const.SUPPORTED_BROWSERS)

    def get_random_desktop_fingerprint(self):
        try:
            return self.all_desktop_profiles().order_by('?')[0]
        except Exception:
            raise exceptions.DJSpooferError('No Desktop Fingerprints Exist')


class ProxyManager(models.Manager):

    def create_general_proxy(self, *args, **kwargs):
        return self.create(mode=const.ProxyModes.GENERAL.value, *args, **kwargs)

    def create_rotating_proxy(self, *args, **kwargs):
        return self.create(mode=const.ProxyModes.ROTATING.value, *args, **kwargs)

    def create_sticky_proxy(self, *args, **kwargs):
        return self.create(mode=const.ProxyModes.STICKY.value, *args, **kwargs)

    def get_rotating_proxy(self):
        q_filter = Q(mode=const.ProxyModes.ROTATING.value)
        try:
            return super().get_queryset().filter(q_filter)[0]
        except IndexError:
            raise exceptions.DJSpooferError('No rotating proxy is available')

    def get_sticky_proxy(self):
        with transaction.atomic():
            q = Q(mode=const.ProxyModes.STICKY.value)
            q &= (Q(last_used__lt=timezone.now() - F('cooldown')) | Q(last_used=None))

            try:
                sticky_proxy = super().get_queryset().select_for_update(skip_locked=True).order_by(
                    F('last_used').asc(nulls_first=True)).filter(q)[0]
            except IndexError:
                raise exceptions.DJSpooferError('No sticky proxy is available')

            sticky_proxy.set_last_used()
            return sticky_proxy

    def get_all_urls(self):
        return super().get_queryset().values_list('url', flat=True)



# Generated by Django 3.2.12 on 2022-04-06 00:39

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('djspoofer', '0011_auto_20220406_0036'),
    ]

    operations = [
        migrations.AddField(
            model_name='fingerprint',
            name='proxy',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='fingerprints', to='djspoofer.proxy'),
        ),
    ]
